;;; patcher-cmtcmd.el --- Commit Command buffers

;; Copyright (C) 2010, 2011 Didier Verna

;; Author:        Didier Verna <didier@lrde.epita.fr>
;; Maintainer:    Didier Verna <didier@lrde.epita.fr>
;; Created:       Sun Apr  4 18:44:08 2010
;; Last Revision: Sun Dec  4 15:54:31 2011
;; Keywords:      maint


;; This file is part of Patcher.

;; Patcher is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License version 3,
;; as published by the Free Software Foundation.

;; Patcher is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with this program; if not, write to the Free Software
;; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


;;; Commentary:

;; Contents management by FCM version 0.1.


;;; Code:

(require 'cl)

(eval-when-compile (require 'patcher-cutil))
(require 'patcher-util)
(require 'patcher-face)
(require 'patcher-project)
(require 'patcher-instance)


(defgroup patcher-cmtcmd nil
  "Patcher settings for commit command buffers."
  :group 'patcher)



;; ==========================================================================
;; Utilities
;; ==========================================================================

(defun patcher-cmtcmd-init (command)
  ;; Initialize the current commit command buffer with COMMAND.
  (insert-string
   (substitute-command-keys
    "\
# Edit / confirm the commit command below.\n\
# - `\\[patcher-cmtcmd-commit]' to commit the project,\n\
# - `\\[patcher-cmtcmd-cancel]' to cancel the commit,\n\
# - `\\[patcher-cmtcmd-kill]' to kill the project.\n\n"))
  (insert-string command))




;; ==========================================================================
;; Patcher CmtCmd fontification
;; ==========================================================================

(defcustom patcher-cmtcmd-font-lock-keywords
  '(("^\\s-*#.*" . patcher-comment-face)
    ("`\\([^'\n]+\\)'" (1 patcher-reference-face prepend)))
  "*Font lock keywords for commit command buffers."
  :group 'patcher-cmtcmd
  :type 'sexp)




;; ==========================================================================
;; Patcher CmtCmd mode
;; ==========================================================================

(defun patcher-cmtcmd-kill ()
  "Kill the project related to the current commit command buffer."
  (interactive)
  (patcher-kill-project patcher-project))

(defun patcher-cmtcmd-cancel ()
  "Cancel the current commit command."
  (interactive)
  (pop-window-configuration))

(defun patcher-cmtcmd-commit ()
  (interactive)
  "Commit project with the command in current buffer.
Comment lines are skipped. Other lines are concatenated together."
  (patcher-detect-committed-project patcher-project)
  (goto-char (point-min))
  (let ((command ""))
    (while (not (eobp))
      (unless (looking-at "[ \t]*#")
	(setq command (concat command
			      " "
			      (buffer-substring (point) (point-at-eol)))))
      (forward-line))
    (patcher-condition-case nil
	(patcher-commit-project patcher-project command)
      (commit
       (if (interactive-p)
	   ;; we come from the CmtCmd buffer.
	   (progn
	     (display-buffer (patcher-project-process-buffer patcher-project)
			     t)
	     (beep)
	     (patcher-message "\
Error during commit. Type \\[patcher-cmtcmd-commit] to try again."))
	 ;; Otherwise, propagate to the caller.
	 (patcher-error 'commit command))))))

(defcustom patcher-cmtcmd-mode-hook nil
  "*Hook to run after setting up Patcher CmtCmd mode."
  :group 'patcher-cmtcmd
  :type 'hook)

(defvar patcher-cmtcmd-mode-map
  (let ((map (make-sparse-keymap)))
    (define-key map [(control c) (control p) c] 'patcher-cmtcmd-commit)
    (define-key map [(control c) (control c)] 'patcher-cmtcmd-commit)
    (define-key map [(control c) (control z)] 'patcher-cmtcmd-cancel)
    (define-key map [(control c) (control k)] 'patcher-cmtcmd-kill)
    map))

(defun patcher-cmtcmd-mode ()
  "Major mode for Patcher commit command management.
You're not supposed to use this mode manually, unless you know what you're
doing.

The following commands are available in a CmtCmd buffer:
\\{patcher-cmtcmd-mode-map}"
  (interactive)
  (kill-all-local-variables)
  (setq major-mode 'patcher-cmtcmd)
  (setq mode-name "Patcher-CmtCmd")
  (use-local-map patcher-cmtcmd-mode-map)
  (setq font-lock-keywords patcher-cmtcmd-font-lock-keywords)
  (run-hooks 'patcher-cmtcmd-mode-hook))


(provide 'patcher-cmtcmd)

;;; patcher-cmtcmd.el ends here
